package nexusclient

import (
	"encoding/json"
	"errors"

	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/common/params"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/common/template"
	u "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/utils"
	"github.com/go-resty/resty/v2"
	"github.com/google/uuid"
)

func (nc *NexusClient) TemplateDel(tplID string) error {
	resp, err := nc.client.R().Delete("/templates/" + tplID)
	if err != nil {
		return err
	}
	if resp.IsSuccess() {
		return nil
	} else {
		return errorToMsg(resp)
	}
}

func (nc *NexusClient) TemplateCreate(vmID uuid.UUID, name, access string) (*template.TemplateSerialized, error) {
	p := &params.TplCreate{VMID: vmID, Name: name, Access: access}
	resp, err := nc.client.R().SetBody(p).Post("/templates/vm")
	if err != nil {
		return nil, err
	}
	if resp.IsSuccess() {
		var tpl template.TemplateSerialized
		if err := json.Unmarshal(resp.Body(), &tpl); err != nil {
			return nil, err
		}
		return &tpl, nil
	} else {
		return nil, errorToMsg(resp)
	}
}

func (nc *NexusClient) GetTemplates() ([]template.TemplateSerialized, error) {
	resp, err := nc.client.R().Get("/templates")
	if err != nil {
		return nil, err
	}

	if resp.IsSuccess() {
		templates, err := deserializeTemplates(resp)
		if err != nil {
			return nil, err
		}
		return templates, nil
	} else {
		return nil, errorToMsg(resp)
	}
}

func (nc *NexusClient) GetTemplate(uuid string) (*template.TemplateSerialized, error) {
	resp, err := nc.client.R().Get("/templates/" + uuid)
	if err != nil {
		return nil, err
	}

	if resp.IsSuccess() {
		template, err := deserializeTemplate(resp)
		if err != nil {
			return nil, err
		}
		return template, nil
	} else {
		return nil, errorToMsg(resp)
	}
}

func (nc *NexusClient) TemplateEdit(tplID string, p params.TplEdit) (*template.TemplateSerialized, error) {
	resp, err := nc.client.R().SetBody(p).Put("/templates/" + tplID)
	if err != nil {
		return nil, err
	}
	if resp.IsSuccess() {
		var tpl template.TemplateSerialized
		if err := json.Unmarshal(resp.Body(), &tpl); err != nil {
			return nil, err
		}
		return &tpl, nil
	} else {
		return nil, errorToMsg(resp)
	}
}

func (nc *NexusClient) TemplateExportDisk(tplID, outputFile string) error {
	resp, err := nc.client.R().SetOutput(outputFile).Get("/templates/" + tplID + "/disk")
	if err != nil {
		return err
	} else {
		if resp.IsSuccess() {
			return nil
		} else {
			errorMsg, _ := u.FileToString(outputFile)
			return errors.New(errorToMsg(resp).Error() + ": " + errorMsg)
		}
	}
}

// Deserialize a list of templates from an http response (no filtering).
func deserializeTemplates(resp *resty.Response) ([]template.TemplateSerialized, error) {
	templates := []template.TemplateSerialized{}
	if err := json.Unmarshal(resp.Body(), &templates); err != nil {
		return nil, err
	}
	return templates, nil
}

// Deserialize a single template from an http response.
func deserializeTemplate(resp *resty.Response) (*template.TemplateSerialized, error) {
	tpl := template.TemplateSerialized{}
	if err := json.Unmarshal(resp.Body(), &tpl); err != nil {
		return nil, err
	}
	return &tpl, nil
}
