package utils

import (
	"encoding/csv"
	"errors"
	"io"
	"os"
	"strconv"

	g "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/globals"
)

// Returns the specified column, as a slice of strings, from a CSV file.
// Columns are numbered starting at 0.
func ReadCSVColumn(csvFile string, column int) ([]string, error) {
	file, err := os.Open(csvFile)
	if err != nil {
		return nil, errors.New("Error: " + err.Error())
	}
	defer file.Close()
	reader := csv.NewReader(file)
	reader.Comma = g.CsvFieldSeparator // specify the column separator
	reader.Comment = '#'               // lines starting with this symbol are to be ignored

	var entries []string
	for {
		record, err := reader.Read()
		if err == io.EOF {
			break
		}
		if err != nil {
			return nil, errors.New("Failed reading " + err.Error())
		}
		colCount := len(record)
		if column >= colCount {
			return nil, errors.New("Failed reading \"" + csvFile + "\": column out of bound (detected " + strconv.Itoa(colCount) + " columns)")
		}
		entries = append(entries, record[column])
	}

	return entries, nil
}
