package cmdUser

import (
	"encoding/csv"
	"io"
	"os"

	g "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/globals"
	nc "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/nexusclient"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/utils"
	u "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/nexush/utils"
)

type Del struct {
	Name string
}

func (cmd *Del) GetName() string {
	return cmd.Name
}

func (cmd *Del) GetDesc() []string {
	return []string{
		"Deletes one or more users.",
		"Requires USER_DESTROY user capability."}
}

func (cmd *Del) PrintUsage() {
	for _, desc := range cmd.GetDesc() {
		u.PrintlnErr(desc)
	}
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("USAGE: " + cmd.GetName() + " email [email ...]")
	u.PrintlnErr("       " + cmd.GetName() + " file.csv")
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("file.csv    1-column CSV file containing emails of users to delete.")
}

func (cmd *Del) Run(client *nc.NexusClient, args []string) int {
	argc := len(args)
	if argc < 1 {
		cmd.PrintUsage()
		return 1
	}

	statusCode := 0

	if argc == 1 && !utils.IsEmail(args[0]) {
		// Single argument and it's a CSV file

		csvFile := args[0]

		if utils.IsEmail(csvFile) {
			cmd.PrintUsage()
			return 1
		}

		file, err := os.Open(csvFile)
		if err != nil {
			u.PrintlnErr("Error: " + err.Error())
			return 1
		}
		defer file.Close()
		reader := csv.NewReader(file)
		reader.Comma = g.CsvFieldSeparator // specify the column separator
		reader.Comment = '#'               // lines starting with this symbol are to be ignored
		line := 0
		for {
			line += 1
			columns, err := reader.Read()
			if err == io.EOF {
				break
			}
			if err != nil {
				u.PrintlnErr("Failed reading " + err.Error())
				statusCode = 1
				continue
			}

			columnCount := len(columns)
			if columnCount != 1 {
				u.PrintlnErr("Failed reading record on line ", line, ": expecting 1 column")
				statusCode = 1
				continue
			}

			email := columns[0]
			if !utils.IsEmail(email) {
				u.PrintlnErr("Failed reading record on line ", line, ": ", email, " is not a valid email")
				statusCode = 1
				continue
			}

			if err := client.UserDelete(email); err != nil {
				u.PrintlnErr(err)
				statusCode = 1
			} else {
				u.Println("Successfully deleted user " + email)
			}
		}
	} else {
		// Argument(s) is/are email addresses

		// Iterates through each user (email) to delete
		for i := range args {
			email := args[i]
			if err := client.UserDelete(email); err != nil {
				u.PrintlnErr(err)
				statusCode = 1
			} else {
				u.Println("Successfully deleted user " + email)
			}
		}
	}

	return statusCode
}
