package cmdVM

import (
	nc "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/nexusclient"
	u "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/nexush/utils"
)

type ExportDir struct {
	Name string
}

func (cmd *ExportDir) GetName() string {
	return cmd.Name
}

func (cmd *ExportDir) GetDesc() []string {
	return []string{
		"Exports one or more VMs' directory into one or more compressed archives.",
		"If not the VM's owner: requires VM_READFS VM access capability or VM_READFS_ANY user capability."}
}

func (cmd *ExportDir) PrintUsage() {
	for _, desc := range cmd.GetDesc() {
		u.PrintlnErr(desc)
	}
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("USAGE: ", cmd.GetName(), " [ID ...] [regex ...] vmDir")
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	const usage string = `vmDir    Directory (in the VM) to export into a compressed archive.
Remark: one archive, named after the VM's name, is created per VM.`
	u.PrintlnErr(usage)
	u.PrintlnErr("")
	printRegexUsageDetails()
}

func (cmd *ExportDir) Run(client *nc.NexusClient, args []string) int {
	argc := len(args)
	if argc < 2 {
		cmd.PrintUsage()
		return 1
	}

	dir := args[argc-1]

	vms, err := getFilteredVMs(client.GetExportDirVMs, args[:argc-1])
	if err != nil {
		u.PrintlnErr(err)
		return 1
	}

	if len(vms) == 0 {
		u.PrintlnErr("Error: VM(s) not found.")
		return 1
	}

	statusCode := 0

	for _, vm := range vms {
		outputFile, err := client.VMExportDir(vm.Name, vm.ID.String(), dir)
		if err != nil {
			u.PrintlnErr(err)
			statusCode = 1
		} else {
			u.Println("Exported " + dir + " from VM \"" + vm.Name + "\" into " + outputFile)
		}
	}

	return statusCode
}
