package exec

import (
	"errors"
	"fmt"
	"os/exec"
	"strconv"
	"strings"

	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/server/logger"
)

const (
	qemuimgBinaryMinVersion = 4
)

var log = logger.GetInstance()

// Checks qemu-img is available and the version is recent enough.
func CheckQemuImg() error {
	output, err := exec.Command(conf.Tools.QemuImg, "--version").Output()
	if err != nil {
		return errors.New(conf.Tools.QemuImg + " is required but not found. On Ubuntu/Debian, it can be installed with \"sudo apt-get install qemu-utils\".")
	}
	out := string(output)
	lines := strings.Split(out, "\n")
	fields := strings.Split(lines[0], " ")
	if len(fields) < 3 {
		return errors.New("failed extracting " + conf.Tools.QemuImg + " version number")
	}
	v := fields[2]
	majorVersionNumber, err := strconv.Atoi(string(v[0]))
	if err != nil {
		return errors.New("failed extracting " + conf.Tools.QemuImg + " version number: " + err.Error())
	}
	if majorVersionNumber < qemuimgBinaryMinVersion {
		return errors.New(conf.Tools.QemuImg + " " + strconv.Itoa(qemuimgBinaryMinVersion) + ".0.0 or newer is required. Found version " + v + ".")
	}
	return nil
}

// Creates a VM disk image baked by a base image (template).
func QemuImgCreate(templateDiskFile, vmDiskFile string) error {
	cmd := exec.Command(conf.Tools.QemuImg, "create", "-F", "qcow2", "-b", templateDiskFile, "-f", "qcow2", vmDiskFile)
	stdoutStderr, err := cmd.CombinedOutput()
	if err != nil {
		output := fmt.Sprintf("[%s]", stdoutStderr)
		log.Error("Failed creating VM disk image: " + output)
		return errors.New("failed creating VM disk image")
	}
	return nil
}

// Rebases a VM overlay image in order to become a standalone image.
func QemuImgRebase(overlayFile string) error {
	cmd := exec.Command(conf.Tools.QemuImg, "rebase", "-b", "", overlayFile)
	stdoutStderr, err := cmd.CombinedOutput()
	if err != nil {
		output := fmt.Sprintf("[%s]", stdoutStderr)
		log.Error("Failed rebasing template disk from VM: " + ": " + output)
		return errors.New("failed rebasing template disk from VM")
	}
	return nil
}
